@@ -1,9 +1,9 @@
-/* knocker version 0.7.1
- * Release date: 24 May 2002
+/* knocker version 0.8.0
+ * Release date: 28 December 2020
  *
- * Project homepage: http://knocker.sourceforge.net
+ * Project homepage: https://knocker.sourceforge.io
  *
- * Copyright 2001,2002 Gabriele Giorgetti <g.gabriele79@genie.it>
+ * Copyright 2001,2020 Gabriele Giorgetti <g.giorgetti@gmail.com>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,10 +32,6 @@
 #include "knocker_conf.h"
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 static int _isblank (int c)
 {
   if (c == '\t')
@@ -48,10 +44,6 @@ static int _isblank (int c)
 }
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 static int _isno (char *answer)
 {
   if (!strcmp (answer, "N"))
@@ -69,10 +61,7 @@ static int _isno (char *answer)
   return 0;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static int _isyes (char *answer)
 {
 
@@ -104,10 +93,6 @@ static int _isyes (char *answer)
 }
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 static char *_get_answer (char *answer)
 {
   char *tmp = malloc (KNOCKER_CONF_MAX_LEN);
@@ -135,10 +120,7 @@ static char *_get_answer (char *answer)
   return answer;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static int _iscommented (char *line, char comment)
 {
   int first_char = 0;
@@ -170,10 +152,7 @@ static int _iscommented (char *line, char comment)
   return 0;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static int _isvalid (char *line, char token)
 {
   /* Check for the  KNOCKER_OPTION_TOKEN  char */
@@ -183,10 +162,7 @@ static int _isvalid (char *line, char token)
   return 1;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static int _fgetline (FILE * fp, char *line, int maxlen)
 {
   int i = 0;
@@ -215,10 +191,7 @@ static int _fgetline (FILE * fp, char *line, int maxlen)
   return (i);
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static int _getoptval (char *line, char *opt, char *value)
 {
   char *tmpp = malloc (strlen (line));
@@ -234,7 +207,7 @@ static int _getoptval (char *line, char *opt, char *value)
 
   if (!_isblank (*tmpp) || *tmpp != KNOCKER_OPTION_TOKEN)
     {
-      free (tmpp);
+      /* free(tmpp); this cause knocker to segfault */
       return 0;
     }
 
@@ -254,10 +227,6 @@ static int _getoptval (char *line, char *opt, char *value)
 }
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 int _get_color_by_opt (char *opt_value)
 {
   int color;
@@ -284,10 +253,6 @@ int _get_color_by_opt (char *opt_value)
   return color;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 int _get_color_attrib_by_opt (char *opt_value)
 {
   int attrib;
@@ -347,10 +312,6 @@ void _set_color_conf (char *opt_value, int n, int attr)
     }
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 void knocker_conf_set_default (void)
 {
   KNOCKER_COLOR_1 = KNOCKER_DEFAULT_COLOR_1;
@@ -360,12 +321,11 @@ void knocker_conf_set_default (void)
 }
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 int knocker_conf_parse (void)
 {
+#ifdef DEBUG
+  fprintf (stderr, "debug: function knocker_conf_parse() called.\n");
+#endif
   FILE *fp;
 
   char *conf_file = NULL;
@@ -378,11 +338,19 @@ int knocker_conf_parse (void)
   strcat (conf_file, "/");
   strcat (conf_file, KNOCKER_CONFIG_FILE);
 
+#ifdef DEBUG
+  fprintf (stderr, "debug: conf_file is %s.\n",conf_file);
+#endif
+
   if ((fp = fopen (conf_file, "r")) == NULL)
     {
       /* fprintf (stderr, "couldn't read config file %s\n", conf_file); */
       free (conf_file);
       knocker_conf_set_default ();      /* run with the default configuration */
+#ifdef DEBUG
+  fprintf (stderr, "debug: function knocker_conf_parse() returns -1.\n");
+#endif
+
       return -1;
     }
 
@@ -431,18 +399,21 @@ int knocker_conf_parse (void)
 
 
   fclose (fp);
-  free (conf_file);
-  free (linebuff);
-  free (opt_value);
+
+  if (conf_file != NULL)
+    free (conf_file);
+   if (linebuff != NULL)
+    free (linebuff);
+  if (opt_value != NULL)
+    free (opt_value);
+
+#ifdef DEBUG
+  fprintf (stderr, "debug: function knocker_conf_parse() returns 0.\n");
+#endif
 
   return 0;
 }
 
-
-/*
-   ============================================================================
-   ============================================================================
-*/
 int knocker_conf_write_default (void)
 {
   FILE *fp;
@@ -498,10 +469,6 @@ int knocker_conf_write_default (void)
   return 0;
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 int knocker_conf_write (int use_colors, char *color1, char *color2, char *color1_attr, char *color2_attr)
 {
   FILE *fp;
@@ -600,11 +567,6 @@ int knocker_conf_write (int use_colors, char *color1, char *color2, char *color1
   return 0;
 }
 
-
-/*
-   ============================================================================
-   ============================================================================
-*/
 void knocker_conf_configure (void)
 {
   char *buff = NULL;
