@@ -1,9 +1,9 @@
-/* knocker version 0.7.1
- * Release date: 24 May 2002
+/* knocker version 0.8.0
+ * Release date: 28 December 2020
  *
- * Project homepage: http://knocker.sourceforge.net
+ * Project homepage: https://knocker.sourceforge.io
  *
- * Copyright 2001,2002 Gabriele Giorgetti <g.gabriele79@genie.it>
+ * Copyright 2001,2020 Gabriele Giorgetti <g.giorgetti@gmail.com>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,11 +37,18 @@
 #include "knocker_conf.h"
 #include "knocker_core.h"
 #include "knocker_time.h"
-#include "knocker_led.h"
 #include "knocker_services.h"
 
 #include <time.h>
 
+int KNOCKER_USER_COLOR;
+int KNOCKER_COLOR_1;
+int KNOCKER_COLOR_2;
+int KNOCKER_COLOR_1_ATTR;
+int KNOCKER_COLOR_2_ATTR;
+
+knocker_user_t knocker_user;
+
 int total_open_ports = 0;
 int total_scanned_ports = 0;
 
@@ -49,10 +56,6 @@ double elapsed_time;
 
 knocker_core_portscan_data_t pscan_data;
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 static void init (int argc, char *argv[])
 {
   knocker_core_init ();
@@ -67,12 +70,12 @@ static void init (int argc, char *argv[])
 
   knocker_args_init (&knocker_args, FALSE, FALSE, TRUE);
 
-  knocker_args_parse (&knocker_args, argc, argv);
-
   if (knocker_conf_parse () < 0)
     /* write a default conf file for the user if it is not already there. */
     knocker_conf_write_default ();
 
+  knocker_args_parse (&knocker_args, argc, argv);
+
   knocker_core_init_portscan_data (&pscan_data);
 
   if (knocker_args.fency)
@@ -96,10 +99,7 @@ static void init (int argc, char *argv[])
     }
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static void quit (void)
 {
   knocker_log_close ();
@@ -119,17 +119,12 @@ static void quit (void)
     }
 #endif
 
-  /* knocker_led_reset(); Not ready for leds Yet */
-
   knocker_term_reset ();
 
   knocker_core_quit ();
 }
 
-/*
-   ============================================================================
-   ============================================================================
-*/
+
 static void resolve (void)
 {
   if (knocker_core_resolve_host (&pscan_data, knocker_args.hname) == NULL)
@@ -149,25 +144,17 @@ static void resolve (void)
 
 
 
-
-/*
-   ============================================================================
-   ============================================================================
-*/
 static void portscan (void)
 {
 
   knocker_term_fflush (stdout);
 
-
   if (knocker_args.port != 0)
     {
 
       knocker_output_ports_info (knocker_args.port, knocker_args.port);
       knocker_log_ports_info (knocker_args.port, knocker_args.port);
 
-      knocker_user_write_lastscan (&knocker_user, &knocker_args);
-
       knocker_time_start_timer ();
 
       if (knocker_core_portscan_tcp_connnect (&pscan_data, knocker_args.port) == PORT_OPEN)
@@ -198,8 +185,6 @@ static void portscan (void)
       knocker_output_ports_info (knocker_args.sport, knocker_args.eport);
       knocker_log_ports_info (knocker_args.sport, knocker_args.eport);
 
-      knocker_user_write_lastscan (&knocker_user, &knocker_args);
-
       knocker_time_start_timer ();
 
       for (port = knocker_args.sport; port <= knocker_args.eport; port++)
@@ -222,12 +207,10 @@ static void portscan (void)
 
               knocker_output_open_port (port, knocker_core_last_service);
               knocker_log_open_port (port, knocker_core_last_service);
-
             }
         }
 
       elapsed_time = knocker_time_get_ticks ();
-
       total_scanned_ports = port;
       return;
     }
@@ -239,8 +222,6 @@ static void portscan (void)
     {
       int port = knocker_args.sport;
 
-      knocker_user_write_lastscan (&knocker_user, &knocker_args);
-
       knocker_time_start_timer ();
 
       if (knocker_core_portscan_tcp_connnect (&pscan_data, port) == PORT_OPEN)
@@ -254,7 +235,6 @@ static void portscan (void)
               knocker_core_services_db_get_service (port, PROTO_TCP);
             }
 #endif
-
           knocker_output_open_port (port, knocker_core_last_service);
           knocker_log_open_port (port, knocker_core_last_service);
         }
@@ -269,8 +249,6 @@ static void portscan (void)
     {
       int port = 0;             /* avaoid unitialization warning */
 
-      knocker_user_write_lastscan (&knocker_user, &knocker_args);
-
       knocker_time_start_timer ();
 
       for (port = knocker_args.sport; port >= knocker_args.eport; port--)
@@ -291,10 +269,8 @@ static void portscan (void)
               knocker_output_open_port (port, knocker_core_last_service);
               knocker_log_open_port (port, knocker_core_last_service);
             }
-
         }
       elapsed_time = knocker_time_get_ticks ();
-
       total_scanned_ports = knocker_args.sport - port;
       return;
     }
@@ -303,18 +279,10 @@ static void portscan (void)
     {
       int port;
 
-      knocker_user_write_lastscan (&knocker_user, &knocker_args);
-
       knocker_time_start_timer ();
 
       for (port = knocker_args.sport; port <= knocker_args.eport; port++)
         {
-          /* Not ready yet for these stuffs
-             knocker_led_sequence();
-             usleep(10000); / usleep is deprecated, use nanosleep instead 
-             knocker_time_delay(10);
-           */
-
           if (knocker_core_portscan_tcp_connnect (&pscan_data, port) == PORT_OPEN)
             {
               total_open_ports++;
@@ -342,12 +310,6 @@ static void portscan (void)
 }
 
 
-
-
-/*
-   ============================================================================
-   ============================================================================
-*/
 static void results (void)
 {
   char timestr[80];
@@ -360,20 +322,12 @@ static void results (void)
 }
 
 
-/*
-   ============================================================================
-   ============================================================================
-*/
 int main (int argc, char *argv[])
 {
   init (argc, argv);
-
   atexit (quit);
-
   resolve ();
-
   portscan ();
-
   results ();
 
   return (EXIT_SUCCESS);
