@@ -1,9 +1,9 @@
-/* knocker version 0.7.1
- * Release date: 24 May 2002
+/* knocker version 0.8.0
+ * Release date: 28 December 2020
  *
- * Project homepage: http://knocker.sourceforge.net
+ * Project homepage: https://knocker.sourceforge.io
  *
- * Copyright 2001,2002 Gabriele Giorgetti <g.gabriele79@genie.it>
+ * Copyright 2001,2020 Gabriele Giorgetti <g.giorgetti@gmail.com>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
+
 #ifndef _KNOCKER_SERVICES_H_
 #define _KNOCKER_SERVICES_H_
 
@@ -29,9 +30,9 @@
 
 #define SERVICES_DB_TOTAL_SERVICES 30000 /* was erroneously 231 */ 
 
-int knocker_core_services_db_init;
+extern int knocker_core_services_db_init;
 
-char *knocker_core_services_db[SERVICES_DB_TOTAL_SERVICES];
+extern char *knocker_core_services_db[SERVICES_DB_TOTAL_SERVICES];
 
 int  init_knocker_core_services_db ( void );
 void free_knocker_core_services_db ( void );
