@@ -1,9 +1,9 @@
-/* knocker version 0.7.1
- * Release date: 24 May 2002
+/* knocker version 0.8.0
+ * Release date: 28 December 2020
  *
- * Project homepage: http://knocker.sourceforge.net
+ * Project homepage: https://knocker.sourceforge.io
  *
- * Copyright 2001,2002 Gabriele Giorgetti <g.gabriele79@genie.it>
+ * Copyright 2001,2020 Gabriele Giorgetti <g.giorgetti@gmail.com>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@ typedef struct {
   int    super;  /* user is root or not */
 } knocker_user_t;
 
-knocker_user_t knocker_user;
+extern knocker_user_t knocker_user;
 
 int   knocker_user_is_root ( void );
 char *knocker_user_get_username ( void );
@@ -39,9 +39,4 @@ char *knocker_user_get_userhome ( void );
 int  knocker_user_init (knocker_user_t *user);
 void knocker_user_free (knocker_user_t *user);
 
-int knocker_user_read_lastscan  (knocker_user_t *user, knocker_args_t *args);
-int knocker_user_write_lastscan (knocker_user_t *user, knocker_args_t *args);
-
-int knocker_user_read_lasthost (knocker_user_t *user, knocker_args_t *args);
-
 #endif /* _KNOCKER_USER_H_ */
